<?php

declare(strict_types=1);

namespace mirzaev\gavno\models;

// Фреймворк ArangoDB
use mirzaev\arangodb\collection,
	mirzaev\arangodb\document;

// Фреймворк для сайта ipinfo.io
use ipinfo\ipinfo\IPinfo;

// Библиотека для ArangoDB
use ArangoDBClient\Document as _document;

// Встроенные библиотеки
use exception;

/**
 * Счётчик просмотров
 *
 * @package mirzaev\gavno\models
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
class views extends core
{
	/**
	 * Коллекция
	 */
	final public const COLLECTION = 'views';

	public static function increase(array &$errors = []): ?bool
	{
		try {
			if (collection::init(static::$arangodb->session, self::COLLECTION))
				if ($_SERVER['HTTP_USER_AGENT'] === 'nginx-ssl early hints') return null;
				else if (document::write(static::$arangodb->session, self::COLLECTION, [
					'ip' => $_SERVER['REMOTE_ADDR'] ?? null,
					'x-forwarded-for' => $_SERVER['HTTP_X_FORWARDED_FOR'] ?? null,
					'referer' => $_SERVER['HTTP_REFERER'] ?? null,
					'useragent' => $_SERVER['HTTP_USER_AGENT'] ?? null
				] + (array) (new IPinfo(require '../settings/ipinfo.php'))->getDetails($_SERVER['HTTP_X_FORWARDED_FOR'] ?? $_SERVER['REMOTE_ADDR']))) return true;
				else throw new exception('Не удалось создать аккаунт');
			else throw new exception('Не удалось инициализировать коллекцию');
		} catch (exception $e) {
			// Запись в реестр ошибок
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		return false;
	}

	public static function day(array &$errors = []): ?int
	{
		try {
			if (collection::init(static::$arangodb->session, self::COLLECTION))
				return collection::search(static::$arangodb->session, sprintf(
					<<<AQL
						return COUNT_DISTINCT(
							FOR d IN %s
							FILTER d.created >= %d
							RETURN d['x-forwarded-for']
						)
					AQL,
					views::COLLECTION,
					time() - 86400
				));
			else throw new exception('Не удалось инициализировать коллекцию');
		} catch (exception $e) {
			// Запись в реестр ошибок
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		return null;
	}

	public static function week(array &$errors = []): ?int
	{
		try {
			if (collection::init(static::$arangodb->session, self::COLLECTION))
				return collection::search(static::$arangodb->session, sprintf(
					<<<AQL
						return COUNT_DISTINCT(
							FOR d IN %s
							FILTER d.created >= %d
							RETURN d['x-forwarded-for']
						)
					AQL,
					views::COLLECTION,
					time() - 604800
				));
			else throw new exception('Не удалось инициализировать коллекцию');
		} catch (exception $e) {
			// Запись в реестр ошибок
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		return null;
	}

	public static function month(array &$errors = []): ?int
	{
		try {
			if (collection::init(static::$arangodb->session, self::COLLECTION))
				return collection::search(static::$arangodb->session, sprintf(
					<<<AQL
						return COUNT_DISTINCT(
							FOR d IN %s
							FILTER d.created >= %d
							RETURN d['x-forwarded-for']
						)
					AQL,
					views::COLLECTION,
					time() - 2592000
				));
			else throw new exception('Не удалось инициализировать коллекцию');
		} catch (exception $e) {
			// Запись в реестр ошибок
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		return null;
	}

	public static function all(array &$errors = []): ?int
	{
		try {
			if (collection::init(static::$arangodb->session, self::COLLECTION))
				return collection::search(static::$arangodb->session, sprintf(
					<<<AQL
						return COUNT_DISTINCT(
							FOR d IN %s
							RETURN d['x-forwarded-for']
						)
					AQL,
					views::COLLECTION
				));
			else throw new exception('Не удалось инициализировать коллекцию');
		} catch (exception $e) {
			// Запись в реестр ошибок
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		return null;
	}

	public static function last(int $amount = 10, array &$errors = []): ?array
	{
		try {
			if (collection::init(static::$arangodb->session, self::COLLECTION)) {
				// Инициализирована коллекция

				// Поиск последних просмотров
				$response = @collection::search(static::$arangodb->session, sprintf(
					<<<AQL
						FOR ip in (
							FOR d IN views
								SORT d.created DESC
								RETURN DISTINCT d['x-forwarded-for'] == null ? d.ip : d['x-forwarded-for']
						)
						LIMIT 10
						RETURN (
							FOR d IN views
								SORT d.created DESC
								FILTER d['x-forwarded-for'] == ip || d.ip == ip
								LIMIT 1
								RETURN d
						)
					AQL,
					views::COLLECTION,
					$amount
				));

				// Инициализация буфера обработанных последних просмотров
				$buffer = [];

				// Универсализация
				if ($response instanceof _document) $response = [$response];

				// Обработка последних просмотров
				foreach ($response ?? [] as $view) $buffer[] = $view->getAll()[0];

				return $buffer;
			} else throw new exception('Не удалось инициализировать коллекцию');
		} catch (exception $e) {
			// Запись в реестр ошибок
			$errors[] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];

		}

		return null;
	}
}
